var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhCalendar', function(ZhUtil, i18nEngine, ScreenService) {
	return {
		restrict: 'A',
		link: function($scope, element) {

			var controlFormat = "YYYY-MM-DD";
			var controlFormatMonth = "YYYY-MM";
			var controlFormatYear = "YYYY";

			var MODE_DAY = "day";
			var MODE_MONTH = "month";
			var MODE_YEAR = "year";

			var UNIT_DAY = "day";
			var UNIT_WEEK = "week";
			var UNIT_MONTH = "month";
			var UNIT_YEAR = "year";

			var widget = $scope.widget;
			widget.selectedDates = [];

			$scope.calendarMode = widget.calendarMode || MODE_DAY;
			$scope.today = moment().startOf(UNIT_DAY);
			$scope.dateFormat = $scope.widget.dateFormat || "L";

			if (widget.field) {
				if (widget.field.currentDate) {
					$scope.currentDate = moment(widget.field.currentDate, widget.field.dateFormat).startOf(UNIT_MONTH).startOf(UNIT_DAY);
				} else if (widget.field.defaultValue) {
					$scope.currentDate = moment(ZhUtil.evaluate(widget.field.defaultValue), $scope.dateFormat).startOf(UNIT_MONTH).startOf(UNIT_DAY);
				}
			} else {
				if (widget.currentDate) {
					$scope.currentDate = moment(widget.currentDate, $scope.dateFormat).startOf(UNIT_MONTH).startOf(UNIT_DAY);
				}
			}

			if (!$scope.currentDate || !$scope.currentDate.isValid()) {
				var lastDate;
				if (widget.getMonth && widget.getYear) {
					lastDate = moment(widget.getYear() + "/" + (widget.getMonth() + 1), "YYYY/MM");
				}
				$scope.currentDate = lastDate && lastDate.isValid() ? lastDate : moment().startOf(UNIT_MONTH).startOf(UNIT_DAY);
			}

			$scope.todayText = $scope.widget.todayText || i18nEngine.i18n("Today");
			$scope.weekDays = $scope.widget.weekDays || moment.weekdaysShort();
			$scope.monthNames = $scope.widget.monthNames || moment.months();

			$scope.maxDate = "";
			$scope.momentMaxDate = "";

			$scope.minDate = "";
			$scope.momentMinDate = "";

			widget.setDateAsSelected = function(dateStr, dateFormat) {
				widget.selectedDates.push(moment(dateStr, dateFormat || widget.dateFormat || controlFormat).format(controlFormat));
			};

			function updateMaxAndMinDates() {
				$scope.maxDate = $scope.widget.maxDate === "today" ? moment().format($scope.dateFormat) : $scope.widget.maxDate;
				$scope.momentMaxDate = moment($scope.maxDate, $scope.dateFormat);

				$scope.minDate = $scope.widget.minDate === "today" ? moment().format($scope.dateFormat) : $scope.widget.minDate;
				$scope.momentMinDate = moment($scope.minDate, $scope.dateFormat);
			}
			updateMaxAndMinDates();



			// Auxiliar functions
			function isBlockedDate(date) {
				var dayMoment = moment(date, $scope.dateFormat);
				return (
					($scope.momentMinDate.isValid() && dayMoment.isBefore($scope.momentMinDate)) ||
					($scope.momentMaxDate.isValid() && dayMoment.isAfter($scope.momentMaxDate))
				);
			}

			function isBlockedMonth(monthDate) {
				var monthMomentBegining = moment(monthDate, controlFormatMonth).startOf(UNIT_MONTH);
				var monthMomentEnd = moment(monthDate, controlFormatMonth).endOf(UNIT_MONTH);
				return (
					($scope.momentMaxDate.isValid() && monthMomentBegining.isAfter($scope.momentMaxDate)) ||
					($scope.momentMinDate.isValid() && monthMomentEnd.isBefore($scope.momentMinDate))
				);
			}

			function isBlockedYear(yearNumber) {
				var yearMomentBegining = moment(yearNumber, controlFormatYear).startOf(UNIT_YEAR);
				var yearMomentEnd = moment(yearNumber, controlFormatYear).endOf(UNIT_YEAR);
				return (
					($scope.momentMaxDate.isValid() && yearMomentBegining.isAfter($scope.momentMaxDate)) ||
					($scope.momentMinDate.isValid() && yearMomentEnd.isBefore($scope.momentMinDate))
				);
			}

			function isSelectedDate(date) {
				return widget.selectedDates.indexOf(date) != -1;
			}

			$scope.isBlockedDate = isBlockedDate;
			$scope.isBlockedMonth = isBlockedMonth;
			$scope.isBlockedYear = isBlockedYear;

			function changeCalendarMode(mode, changeToParameterMode) {
				var newMode = mode;

				if (!changeToParameterMode) {
					switch (mode) {
						case MODE_DAY:
							newMode = MODE_MONTH;
							break;
						case MODE_MONTH:
							newMode = MODE_YEAR;
							break;
					}
				}

				$scope.calendarMode = newMode;
			}

			$scope.changeCalendarMode = changeCalendarMode;

			$scope.getCalendarHeading = function(mode) {
				var heading = "";

				switch (mode) {
					case MODE_YEAR:
						heading = $scope.yearRows ? $scope.yearRows[0].years[0].number + " - " + $scope.yearRows[2].years[3].number : "";
						break;
					case MODE_MONTH:
						heading = $scope.currentDate.format(controlFormatYear);
						break;
					case MODE_DAY:
						heading = $scope.currentDate.format("MMMM") + ' ' + $scope.currentDate.format("YYYY");
						break;
				}

				return heading;
			};



			// Navigation functions
			$scope.navigateCalendar = function(value) {
				switch ($scope.calendarMode) {
					case MODE_YEAR:
						navigateYearMode(value);
						break;
					case MODE_MONTH:
						navigateMonthMode(value);
						break;
					case MODE_DAY:
						navigateDayMode(value);
						break;
				}
			};

			function navigateYearMode(value) {
				$scope.currentDate.add(value * 10, UNIT_YEAR);
			}

			function navigateMonthMode(value) {
				$scope.currentDate.add(value, UNIT_YEAR);
			}

			function navigateDayMode(value) {
				$scope.currentDate.add(value, UNIT_MONTH);
			}



			// Calendar rendering functions
			function buildCalendar(mode) {
				switch (mode) {
					case MODE_YEAR:
						buildYearCalendar();
						break;
					case MODE_MONTH:
						buildMonthCalendar();
						break;
					case MODE_DAY:
						buildDayCalendar();
						break;
				}
			}

			function buildDayCalendar() {
				// Clones the current date to prevent changing it
				var current = $scope.currentDate.clone().utc().startOf(UNIT_WEEK).day(0);

				// Checks if the first day of the month is also the first day of its week so we should put it on the second row of the calendar
				if (current.format(controlFormat) === $scope.currentDate.clone().format(controlFormat)) {
					current.subtract(7, UNIT_DAY);
				}

				// Iterates through the datasource seeking for this month events and builds an object grouping them
				var monthEvents = {};
				if ($scope.widget.dataSource && $scope.widget.dataSource.data) {
					$scope.widget.dataSource.data
						.filter(function(item) {
							var month = moment(item[$scope.widget.dateField], $scope.dateFormat).get(UNIT_MONTH);
							var currMonth = current.get(UNIT_MONTH);
							var isPreviousMonth = month === currMonth;
							var isCurrentMonth = month === ((currMonth + 1) % 12);
							var isNextMonth = month === ((currMonth + 2) % 12);
							return isPreviousMonth || isCurrentMonth || isNextMonth;
						})
						.forEach(function(item) {
							var itemTime = moment(item[$scope.widget.dateField], $scope.dateFormat).startOf(UNIT_DAY).format(controlFormat);

							if (!monthEvents[itemTime]) {
								monthEvents[itemTime] = [];
							}

							monthEvents[itemTime].push(item);
						});
				}

				var displayFormat = ($scope.widget.field && $scope.widget.field.displayFormat) || $scope.dateFormat;
			
				// Start creating week
				$scope.weeks = [];
				for (var i = 0; i < 6; i++) {
					var week = {
						days: []
					};
					for (var j = 0; j < 7; j++) {
						week.days.push({
							day: current.date(),
							date: current.format(displayFormat),
							today: current.format(controlFormat) === $scope.today.format(controlFormat),
							lastMonth: (current.month() < $scope.currentDate.month() && current.year() == $scope.currentDate.year()) || (current.month() > $scope.currentDate.month() && current.year() < $scope.currentDate.year()),
							nextMonth: (current.month() > $scope.currentDate.month() && current.year() == $scope.currentDate.year()) || (current.month() < $scope.currentDate.month() && current.year() > $scope.currentDate.year()),
							events: monthEvents[current.format(controlFormat)] || [],
							selected: isSelectedDate(current.format(controlFormat))
						});
						current.add(1, UNIT_DAY);
					}
					$scope.weeks.push(week);
				}
			}

			function buildMonthCalendar() {
				var auxiliarRow = [];
				$scope.monthRows = [];

				$scope.currentDate.startOf(UNIT_YEAR);
				var year = $scope.currentDate.format(controlFormatYear);

				var shortMonthNames = moment.monthsShort();
				for (var i = 0; i < 12; i++) {
					auxiliarRow.push({
						date: moment("01/" + (i + 1) + "/" + year, "DD/MM/YYYY").format(controlFormatMonth),
						selected: false,
						name: shortMonthNames[i]
					});

					if (auxiliarRow.length == 4) {
						$scope.monthRows.push({
							"months": auxiliarRow
						});
						auxiliarRow = [];
					}
				}
			}

			function buildYearCalendar() {
				var auxiliarRow = [];
				$scope.yearRows = [];

				var currentYear = parseInt($scope.currentDate.format(controlFormatYear));
				var gridInitialYear = Math.floor(currentYear / 10) * 10;

				$scope.currentDate = moment(gridInitialYear, controlFormatYear);
				for (var i = -1; i < 11; i++) {
					auxiliarRow.push({
						number: moment("01/01/" + (gridInitialYear + i), "DD/MM/YYYY").format(controlFormatYear),
						selected: false,
						lastDecade: i == -1,
						nextDecade: i == 10,
					});

					if (auxiliarRow.length == 4) {
						$scope.yearRows.push({
							"years": auxiliarRow
						});
						auxiliarRow = [];
					}
				}
			}



			// Click functions
			$scope.dayClick = function(day) {
				if (day.lastMonth) {
					navigateDayMode(-1);
				} else if (day.nextMonth) {
					navigateDayMode(+1);
				} else if (!isBlockedDate(day.date)) {

					if (widget.dateClick) {
						widget.dateClick();
					}

					if (widget.field) {
						widget.field.currentDate = $scope.currentDate;
					}

					widget.setCurrentRow(day.date);
				}
			};

			$scope.monthClick = function(month) {
				if (!isBlockedMonth(month.date)) {
					$scope.currentDate = moment(month.date, controlFormatMonth);
					changeCalendarMode(MODE_DAY, true);
				}

				if (widget.monthClick) {
					widget.monthClick();
				}
			};

			$scope.yearClick = function(year) {
				if (year.lastDecade) {
					navigateYearMode(-1);
				} else if (year.nextDecade) {
					navigateYearMode(+1);
				} else if (!isBlockedYear(year.number)) {
					$scope.currentDate = moment(year.number, controlFormatYear);
					changeCalendarMode(MODE_MONTH, true);
				}

				if (widget.yearClick) {
					widget.yearClick();
				}
			};

			// Event triggering functions
			function triggerChangeEvents(currentMoment, oldMoment) {
				var monthBeforeChange = oldMoment.month();
				var monthAfterChange = currentMoment.month();

				var yearBeforeChange = oldMoment.year();
				var yearAfterChange = currentMoment.year();

				var decadeBeforeChange = Math.floor(oldMoment.year() / 10) * 10;
				var decadeAfterChange = Math.floor(currentMoment.year() / 10) * 10;

				if (monthBeforeChange != monthAfterChange) {
					triggerMonthChangeEvent();
				}

				if (yearBeforeChange != yearAfterChange) {
					triggerYearChangeEvent();
				}

				if (decadeBeforeChange != decadeAfterChange) {
					triggerDecadeChangeEvent();
				}
			}

			function triggerMonthChangeEvent() {
				if (widget.onChangeMonth) {
					widget.onChangeMonth();
				}
			}

			function triggerYearChangeEvent() {
				if (widget.onChangeYear) {
					widget.onChangeYear();
				}
			}

			function triggerDecadeChangeEvent() {
				if (widget.onChangeDecade) {
					widget.onChangeDecade();
				}
			}



			// Watches
			$scope.$watch("calendarMode", function() {
				buildCalendar($scope.calendarMode);
			});

			$scope.$watch(
				function() {
					return $scope.currentDate._d.getTime();
				},
				function(newTimestamp, oldTimestamp) {
					var currentMoment = moment(new Date(newTimestamp));
					var oldMoment = moment(new Date(oldTimestamp));

					buildCalendar($scope.calendarMode);
					triggerChangeEvents(currentMoment, oldMoment);
				}
			);

			$scope.$watchCollection("widget.selectedDates",
				function() {
					buildCalendar($scope.calendarMode);
				}
			);

			$scope.$watch(
				function() {
					if ($scope.widget.dataSource) {
						return $scope.widget.dataSource.data;
					}
				},
				function() {
					buildCalendar($scope.calendarMode);
				}
			);

			$scope.$watch(
				function() {
					return $scope.widget.minDate;
				},
				function() {
					if ($scope.widget.minDate) {
						updateMaxAndMinDates();
					}
				}
			);

			$scope.$watch(
				function() {
					return $scope.widget.maxDate;
				},
				function() {
					if ($scope.widget.maxDate) {
						updateMaxAndMinDates();
					}
				}
			);

			$scope.widget.getYear = function() {
				return $scope.currentDate.get('year');
			};

			$scope.widget.getMonthName = function() {
				return $scope.monthNames[$scope.currentDate.get('month')];
			};

			$scope.widget.getMonth = function() {
				return $scope.currentDate.get('month');
			};

			$scope.widget.getCurrentDate = function() {
				return $scope.currentDate;
			};

			$scope.widget.setCurrentDate = function(currentDate) {
				if (typeof currentDate === "string") {
					$scope.currentDate = moment(currentDate, $scope.dateFormat);
				} else if (currentDate instanceof Date) {
					$scope.currentDate = moment(currentDate);
				} else {
					$scope.currentDate = currentDate;
				}
				ScreenService.updateTemplate();
			};
			
			var _widget = $scope.widget;
			$scope.$on('$destroy', function() {
				delete _widget.setDateAsSelected;
				delete _widget.getYear;
				delete _widget.getMonthName;
				delete _widget.getMonth;
				delete _widget.getCurrentDate;
			});
		}
	};
});